
import { useState, useEffect } from "react";
import { Card, CardContent, CardHeader, CardTitle } from "@/components/ui/card";
import { 
  Select,
  SelectContent,
  SelectItem,
  SelectTrigger,
  SelectValue,
} from "@/components/ui/select";
import { Input } from "@/components/ui/input";
import { Button } from "@/components/ui/button";
import { CategoryType } from "./ConversionCategories";
import { ArrowRightLeft } from "lucide-react";
import { UnitSearch } from "./UnitSearch";
import { useIsMobile } from "@/hooks/use-mobile";
import { ConversionHistoryItem } from "./ConversionHistory";

interface ConverterProps {
  category: CategoryType;
  onAddToHistory?: (item: ConversionHistoryItem) => void;
  onRestoreFromHistory?: (item: ConversionHistoryItem) => void;
}

interface UnitOption {
  value: string;
  label: string;
  conversionFactor: number;
  isInverse?: boolean; // Added this property to fix the type error
}

export function Converter({ category, onAddToHistory, onRestoreFromHistory }: ConverterProps) {
  const [inputValue, setInputValue] = useState<string>("1");
  const [fromUnit, setFromUnit] = useState<string>("");
  const [toUnit, setToUnit] = useState<string>("");
  const [result, setResult] = useState<string>("");
  const [unitOptions, setUnitOptions] = useState<UnitOption[]>([]);
  const isMobile = useIsMobile();

  // Define unit options based on category
  useEffect(() => {
    let options: UnitOption[] = [];
    
    switch (category) {
      case 'length':
        options = [
          { value: 'meter', label: 'Meter (m)', conversionFactor: 1 },
          { value: 'kilometer', label: 'Kilometer (km)', conversionFactor: 0.001 },
          { value: 'centimeter', label: 'Centimeter (cm)', conversionFactor: 100 },
          { value: 'millimeter', label: 'Millimeter (mm)', conversionFactor: 1000 },
          { value: 'inch', label: 'Inch (in)', conversionFactor: 39.3701 },
          { value: 'foot', label: 'Foot (ft)', conversionFactor: 3.28084 },
          { value: 'yard', label: 'Yard (yd)', conversionFactor: 1.09361 },
          { value: 'mile', label: 'Mile (mi)', conversionFactor: 0.000621371 },
          { value: 'nautical_mile', label: 'Nautical Mile (nmi)', conversionFactor: 0.000539957 },
        ];
        break;
      case 'weight':
        options = [
          { value: 'kilogram', label: 'Kilogram (kg)', conversionFactor: 1 },
          { value: 'gram', label: 'Gram (g)', conversionFactor: 1000 },
          { value: 'milligram', label: 'Milligram (mg)', conversionFactor: 1000000 },
          { value: 'metric_ton', label: 'Metric Ton (t)', conversionFactor: 0.001 },
          { value: 'pound', label: 'Pound (lb)', conversionFactor: 2.20462 },
          { value: 'ounce', label: 'Ounce (oz)', conversionFactor: 35.274 },
          { value: 'stone', label: 'Stone (st)', conversionFactor: 0.157473 },
          { value: 'short_ton', label: 'Short Ton (US)', conversionFactor: 0.00110231 },
          { value: 'long_ton', label: 'Long Ton (UK)', conversionFactor: 0.000984207 },
        ];
        break;
      case 'temperature':
        options = [
          { value: 'celsius', label: 'Celsius (°C)', conversionFactor: 0 }, // Special case
          { value: 'fahrenheit', label: 'Fahrenheit (°F)', conversionFactor: 0 }, // Special case
          { value: 'kelvin', label: 'Kelvin (K)', conversionFactor: 0 }, // Special case
        ];
        break;
      case 'area':
        options = [
          { value: 'square_meter', label: 'Square Meter (m²)', conversionFactor: 1 },
          { value: 'square_kilometer', label: 'Square Kilometer (km²)', conversionFactor: 0.000001 },
          { value: 'square_centimeter', label: 'Square Centimeter (cm²)', conversionFactor: 10000 },
          { value: 'square_millimeter', label: 'Square Millimeter (mm²)', conversionFactor: 1000000 },
          { value: 'square_inch', label: 'Square Inch (in²)', conversionFactor: 1550 },
          { value: 'square_foot', label: 'Square Foot (ft²)', conversionFactor: 10.7639 },
          { value: 'acre', label: 'Acre (ac)', conversionFactor: 0.000247105 },
          { value: 'hectare', label: 'Hectare (ha)', conversionFactor: 0.0001 },
          { value: 'square_mile', label: 'Square Mile (mi²)', conversionFactor: 3.86102e-7 },
        ];
        break;
      case 'volume':
        options = [
          { value: 'cubic_meter', label: 'Cubic Meter (m³)', conversionFactor: 1 },
          { value: 'cubic_kilometer', label: 'Cubic Kilometer (km³)', conversionFactor: 1e-9 },
          { value: 'cubic_centimeter', label: 'Cubic Centimeter (cm³)', conversionFactor: 1e6 },
          { value: 'cubic_millimeter', label: 'Cubic Millimeter (mm³)', conversionFactor: 1e9 },
          { value: 'liter', label: 'Liter (L)', conversionFactor: 1000 },
          { value: 'milliliter', label: 'Milliliter (mL)', conversionFactor: 1e6 },
          { value: 'gallon_us', label: 'Gallon (US)', conversionFactor: 264.172 },
          { value: 'gallon_uk', label: 'Gallon (UK)', conversionFactor: 219.969 },
          { value: 'quart_us', label: 'Quart (US)', conversionFactor: 1056.69 },
          { value: 'quart_uk', label: 'Quart (UK)', conversionFactor: 879.877 },
          { value: 'pint_us', label: 'Pint (US)', conversionFactor: 2113.38 },
          { value: 'pint_uk', label: 'Pint (UK)', conversionFactor: 1759.75 },
          { value: 'cup_us', label: 'Cup (US)', conversionFactor: 4226.75 },
          { value: 'cup_metric', label: 'Cup (Metric)', conversionFactor: 4000 },
          { value: 'fluid_ounce_us', label: 'Fluid Ounce (US)', conversionFactor: 33814 },
          { value: 'fluid_ounce_uk', label: 'Fluid Ounce (UK)', conversionFactor: 35195.1 },
          { value: 'tablespoon_us', label: 'Tablespoon (US)', conversionFactor: 67628 },
          { value: 'tablespoon_metric', label: 'Tablespoon (Metric)', conversionFactor: 66666.7 },
          { value: 'teaspoon_us', label: 'Teaspoon (US)', conversionFactor: 202884 },
          { value: 'teaspoon_metric', label: 'Teaspoon (Metric)', conversionFactor: 200000 },
          { value: 'barrel_oil', label: 'Barrel (Oil)', conversionFactor: 6.29 },
          { value: 'barrel_us', label: 'Barrel (US)', conversionFactor: 8.38641 },
          { value: 'barrel_uk', label: 'Barrel (UK)', conversionFactor: 6.11023 },
        ];
        break;
      case 'time':
        options = [
          { value: 'second', label: 'Second (s)', conversionFactor: 1 },
          { value: 'minute', label: 'Minute (min)', conversionFactor: 0.0166667 },
          { value: 'hour', label: 'Hour (h)', conversionFactor: 0.000277778 },
          { value: 'day', label: 'Day (d)', conversionFactor: 0.0000115741 },
          { value: 'week', label: 'Week (wk)', conversionFactor: 0.00000165344 },
          { value: 'month', label: 'Month (avg)', conversionFactor: 3.8026e-7 },
          { value: 'year', label: 'Year (yr)', conversionFactor: 3.171e-8 },
          { value: 'decade', label: 'Decade', conversionFactor: 3.171e-9 },
          { value: 'century', label: 'Century', conversionFactor: 3.171e-10 },
        ];
        break;
      case 'speed':
        options = [
          { value: 'meter_per_second', label: 'Meter/Second (m/s)', conversionFactor: 1 },
          { value: 'kilometer_per_hour', label: 'Kilometer/Hour (km/h)', conversionFactor: 3.6 },
          { value: 'mile_per_hour', label: 'Mile/Hour (mph)', conversionFactor: 2.23694 },
          { value: 'foot_per_second', label: 'Foot/Second (ft/s)', conversionFactor: 3.28084 },
          { value: 'knot', label: 'Knot (kn)', conversionFactor: 1.94384 },
          { value: 'mach', label: 'Mach (at sea level)', conversionFactor: 0.00293858 },
        ];
        break;
      case 'data':
        options = [
          { value: 'byte', label: 'Byte (B)', conversionFactor: 1 },
          { value: 'kilobyte', label: 'Kilobyte (KB)', conversionFactor: 0.001 },
          { value: 'megabyte', label: 'Megabyte (MB)', conversionFactor: 0.000001 },
          { value: 'gigabyte', label: 'Gigabyte (GB)', conversionFactor: 1e-9 },
          { value: 'terabyte', label: 'Terabyte (TB)', conversionFactor: 1e-12 },
          { value: 'petabyte', label: 'Petabyte (PB)', conversionFactor: 1e-15 },
          { value: 'bit', label: 'Bit (bit)', conversionFactor: 8 },
          { value: 'kibibyte', label: 'Kibibyte (KiB)', conversionFactor: 0.0009765625 },
          { value: 'mebibyte', label: 'Mebibyte (MiB)', conversionFactor: 9.5367e-7 },
          { value: 'gibibyte', label: 'Gibibyte (GiB)', conversionFactor: 9.3132e-10 },
          { value: 'tebibyte', label: 'Tebibyte (TiB)', conversionFactor: 9.0949e-13 },
        ];
        break;
      case 'pressure':
        options = [
          { value: 'pascal', label: 'Pascal (Pa)', conversionFactor: 1 },
          { value: 'kilopascal', label: 'Kilopascal (kPa)', conversionFactor: 0.001 },
          { value: 'bar', label: 'Bar', conversionFactor: 0.00001 },
          { value: 'psi', label: 'PSI', conversionFactor: 0.000145038 },
          { value: 'atmosphere', label: 'Atmosphere (atm)', conversionFactor: 9.86923e-6 },
          { value: 'torr', label: 'Torr', conversionFactor: 0.00750062 },
          { value: 'millimeter_mercury', label: 'mm of Mercury', conversionFactor: 0.00750062 },
          { value: 'inch_mercury', label: 'inch of Mercury', conversionFactor: 0.000295301 },
        ];
        break;
      case 'energy':
        options = [
          { value: 'joule', label: 'Joule (J)', conversionFactor: 1 },
          { value: 'kilojoule', label: 'Kilojoule (kJ)', conversionFactor: 0.001 },
          { value: 'calorie', label: 'Calorie (cal)', conversionFactor: 0.239006 },
          { value: 'kilocalorie', label: 'Kilocalorie (kcal)', conversionFactor: 0.000239006 },
          { value: 'watt_hour', label: 'Watt Hour (Wh)', conversionFactor: 0.000277778 },
          { value: 'kilowatt_hour', label: 'Kilowatt Hour (kWh)', conversionFactor: 2.77778e-7 },
          { value: 'electron_volt', label: 'Electron Volt (eV)', conversionFactor: 6.242e+18 },
          { value: 'british_thermal_unit', label: 'BTU', conversionFactor: 0.000947817 },
          { value: 'therm', label: 'Therm', conversionFactor: 9.4804e-9 },
          { value: 'foot_pound', label: 'Foot-Pound', conversionFactor: 0.737562 },
        ];
        break;
      case 'currency':
        options = [
          { value: 'usd', label: 'US Dollar (USD)', conversionFactor: 1 },
          { value: 'eur', label: 'Euro (EUR)', conversionFactor: 0.91 },
          { value: 'gbp', label: 'British Pound (GBP)', conversionFactor: 0.78 },
          { value: 'jpy', label: 'Japanese Yen (JPY)', conversionFactor: 151.32 },
          { value: 'cad', label: 'Canadian Dollar (CAD)', conversionFactor: 1.36 },
          { value: 'aud', label: 'Australian Dollar (AUD)', conversionFactor: 1.51 },
          { value: 'cny', label: 'Chinese Yuan (CNY)', conversionFactor: 7.23 },
          { value: 'inr', label: 'Indian Rupee (INR)', conversionFactor: 83.52 },
          { value: 'chf', label: 'Swiss Franc (CHF)', conversionFactor: 0.88 },
          { value: 'sgd', label: 'Singapore Dollar (SGD)', conversionFactor: 1.33 },
        ];
        break;
      case 'angle':
        options = [
          { value: 'degree', label: 'Degree (°)', conversionFactor: 1 },
          { value: 'radian', label: 'Radian (rad)', conversionFactor: 0.0174533 },
          { value: 'gradian', label: 'Gradian (grad)', conversionFactor: 1.11111 },
          { value: 'arcminute', label: 'Arcminute (\')', conversionFactor: 60 },
          { value: 'arcsecond', label: 'Arcsecond (\")', conversionFactor: 3600 },
          { value: 'turn', label: 'Turn', conversionFactor: 0.00277778 },
        ];
        break;
      case 'fuel':
        options = [
          { value: 'mpg_us', label: 'Miles/Gallon (US)', conversionFactor: 1 },
          { value: 'mpg_uk', label: 'Miles/Gallon (UK)', conversionFactor: 0.832674 },
          { value: 'km_per_liter', label: 'Kilometer/Liter', conversionFactor: 0.425144 },
          { value: 'liter_per_100km', label: 'Liter/100 km', conversionFactor: 235.215, isInverse: true },
          { value: 'km_per_gallon', label: 'Kilometer/Gallon', conversionFactor: 1.60934 },
        ];
        break;
      case 'power':
        options = [
          { value: 'watt', label: 'Watt (W)', conversionFactor: 1 },
          { value: 'kilowatt', label: 'Kilowatt (kW)', conversionFactor: 0.001 },
          { value: 'megawatt', label: 'Megawatt (MW)', conversionFactor: 0.000001 },
          { value: 'horsepower', label: 'Horsepower (hp)', conversionFactor: 0.00134102 },
          { value: 'btu_per_hour', label: 'BTU/hour', conversionFactor: 3.41214 },
          { value: 'foot_pound_per_second', label: 'Foot-pound/second', conversionFactor: 0.737562 },
        ];
        break;
      case 'cooking':
        // Cooking uses special conversion factors that vary by ingredient
        options = [
          { value: 'cup_us', label: 'Cup (US)', conversionFactor: 1 },
          { value: 'tablespoon_us', label: 'Tablespoon (US)', conversionFactor: 16 },
          { value: 'teaspoon_us', label: 'Teaspoon (US)', conversionFactor: 48 },
          { value: 'fluid_ounce_us', label: 'Fluid Ounce (US)', conversionFactor: 8 },
          { value: 'cup_metric', label: 'Cup (Metric)', conversionFactor: 0.95 },
          { value: 'milliliter', label: 'Milliliter (mL)', conversionFactor: 236.588 },
          { value: 'liter', label: 'Liter (L)', conversionFactor: 0.236588 },
          { value: 'gram_flour', label: 'Gram - Flour', conversionFactor: 125 },
          { value: 'gram_sugar', label: 'Gram - Sugar', conversionFactor: 200 },
          { value: 'gram_rice', label: 'Gram - Rice', conversionFactor: 180 },
        ];
        break;
      case 'shoe-size':
        options = [
          { value: 'us_men', label: 'US Men', conversionFactor: 1 },
          { value: 'us_women', label: 'US Women', conversionFactor: 1.5 },
          { value: 'uk', label: 'UK', conversionFactor: -0.5 },
          { value: 'eu', label: 'EU', conversionFactor: 33 },
          { value: 'cm', label: 'Centimeter (cm)', conversionFactor: 0.3937 * 7.6 + 9.7 },
          { value: 'mondo', label: 'Mondo Point', conversionFactor: 0.3937 * 7.6 + 9.7 },
        ];
        break;
      default:
        options = [];
    }
    
    setUnitOptions(options);
    if (options.length > 0) {
      setFromUnit(options[0].value);
      setToUnit(options.length > 1 ? options[1].value : options[0].value);
    }
  }, [category]);

  // Calculate the conversion
  useEffect(() => {
    if (!fromUnit || !toUnit || !inputValue) {
      setResult('');
      return;
    }

    const numericValue = parseFloat(inputValue);
    if (isNaN(numericValue)) {
      setResult('Invalid input');
      return;
    }

    const fromOption = unitOptions.find(option => option.value === fromUnit);
    const toOption = unitOptions.find(option => option.value === toUnit);

    if (!fromOption || !toOption) {
      setResult('');
      return;
    }

    // Special case for temperature
    if (category === 'temperature') {
      let resultValue: number;
      
      if (fromUnit === 'celsius' && toUnit === 'fahrenheit') {
        resultValue = (numericValue * 9/5) + 32;
      } else if (fromUnit === 'celsius' && toUnit === 'kelvin') {
        resultValue = numericValue + 273.15;
      } else if (fromUnit === 'fahrenheit' && toUnit === 'celsius') {
        resultValue = (numericValue - 32) * 5/9;
      } else if (fromUnit === 'fahrenheit' && toUnit === 'kelvin') {
        resultValue = (numericValue - 32) * 5/9 + 273.15;
      } else if (fromUnit === 'kelvin' && toUnit === 'celsius') {
        resultValue = numericValue - 273.15;
      } else if (fromUnit === 'kelvin' && toUnit === 'fahrenheit') {
        resultValue = (numericValue - 273.15) * 9/5 + 32;
      } else {
        resultValue = numericValue; // Same unit
      }
      
      setResult(resultValue.toFixed(6).replace(/\.?0+$/, ''));
    } 
    // Special case for fuel economy (some use inverse relationships)
    else if (category === 'fuel') {
      const fromIsInverse = (fromOption as any).isInverse;
      const toIsInverse = (toOption as any).isInverse;
      let resultValue: number;
      
      // Convert between normal and inverse units
      if (fromIsInverse && !toIsInverse) {
        resultValue = 100 / (numericValue / fromOption.conversionFactor * toOption.conversionFactor);
      } else if (!fromIsInverse && toIsInverse) {
        resultValue = 100 / (numericValue * fromOption.conversionFactor / toOption.conversionFactor);
      } else if (fromIsInverse && toIsInverse) {
        resultValue = numericValue * toOption.conversionFactor / fromOption.conversionFactor;
      } else {
        // For standard units (not inverse)
        resultValue = numericValue * fromOption.conversionFactor / toOption.conversionFactor;
      }
      
      setResult(resultValue.toFixed(6).replace(/\.?0+$/, ''));
    }
    // Special case for shoe sizes (uses offsets)
    else if (category === 'shoe-size') {
      // Here's the fix for the ternary operator issue
      const usSize = fromUnit === 'us_men' 
        ? numericValue 
        : fromUnit === 'us_women' 
          ? (numericValue - 1.5) 
          : fromUnit === 'uk' 
            ? (numericValue + 0.5) 
            : fromUnit === 'eu' 
              ? ((numericValue - 33) / 1) 
              : fromUnit === 'cm' || fromUnit === 'mondo'
                ? ((numericValue - 9.7) / (0.3937 * 7.6))
                : 0; // Added a default case to complete the ternary operator
      
      let resultValue: number;
      
      if (toUnit === 'us_men') {
        resultValue = usSize;
      } else if (toUnit === 'us_women') {
        resultValue = usSize + 1.5;
      } else if (toUnit === 'uk') {
        resultValue = usSize - 0.5;
      } else if (toUnit === 'eu') {
        resultValue = usSize + 33;
      } else { // cm or mondo
        resultValue = (usSize * (0.3937 * 7.6)) + 9.7;
      }
      
      setResult(resultValue.toFixed(2).replace(/\.?0+$/, ''));
    } 
    // Default case for most conversions
    else {
      // For other units, use the conversion factor
      const baseValue = numericValue / fromOption.conversionFactor;
      const resultValue = baseValue * toOption.conversionFactor;
      
      setResult(resultValue.toFixed(6).replace(/\.?0+$/, ''));
    }
  }, [fromUnit, toUnit, inputValue, unitOptions, category]);
  
  // Add to history when a conversion is successful
  useEffect(() => {
    if (onAddToHistory && result && fromUnit && toUnit && inputValue && result !== 'Invalid input') {
      const fromOption = unitOptions.find(option => option.value === fromUnit);
      const toOption = unitOptions.find(option => option.value === toUnit);
      
      if (fromOption && toOption) {
        const fromLabel = fromOption.label.split(' ')[0];
        const toLabel = toOption.label.split(' ')[0];
        
        onAddToHistory({
          id: crypto.randomUUID(),
          fromValue: inputValue,
          fromUnit: fromLabel,
          toValue: result,
          toUnit: toLabel,
          category,
          timestamp: new Date()
        });
      }
    }
  }, [result, category, fromUnit, toUnit, inputValue, unitOptions, onAddToHistory]);

  const swapUnits = () => {
    const tempUnit = fromUnit;
    setFromUnit(toUnit);
    setToUnit(tempUnit);
  };

  return (
    <Card className="shadow-md">
      <CardHeader className="pb-3">
        <CardTitle className="text-xl capitalize">{category} Converter</CardTitle>
      </CardHeader>
      <CardContent>
        <div className="grid gap-6">
          <div className="space-y-2">
            <label htmlFor="amount" className="text-sm font-medium">Amount</label>
            <Input
              id="amount"
              type="number"
              placeholder="Enter value"
              value={inputValue}
              onChange={(e) => setInputValue(e.target.value)}
              className="font-mono"
            />
          </div>
          
          <div className={`grid ${isMobile ? 'grid-cols-1 gap-4' : 'grid-cols-[1fr,auto,1fr] gap-3'} items-center`}>
            <div className="space-y-2">
              <label className="text-sm font-medium">From</label>
              {isMobile ? (
                <UnitSearch 
                  units={unitOptions}
                  onSelectUnit={setFromUnit}
                  placeholder="Search from unit..."
                />
              ) : (
                <Select value={fromUnit} onValueChange={setFromUnit}>
                  <SelectTrigger>
                    <SelectValue placeholder="Select unit" />
                  </SelectTrigger>
                  <SelectContent>
                    {unitOptions.map((option) => (
                      <SelectItem key={option.value} value={option.value}>
                        {option.label}
                      </SelectItem>
                    ))}
                  </SelectContent>
                </Select>
              )}
            </div>
            
            <button 
              onClick={swapUnits}
              className={`${isMobile ? 'w-full' : 'mt-6'} bg-secondary hover:bg-secondary/80 p-2 rounded-full transition-colors`}
              aria-label="Swap units"
            >
              <ArrowRightLeft className="h-4 w-4 mx-auto" />
            </button>
            
            <div className="space-y-2">
              <label className="text-sm font-medium">To</label>
              {isMobile ? (
                <UnitSearch 
                  units={unitOptions}
                  onSelectUnit={setToUnit}
                  placeholder="Search to unit..."
                />
              ) : (
                <Select value={toUnit} onValueChange={setToUnit}>
                  <SelectTrigger>
                    <SelectValue placeholder="Select unit" />
                  </SelectTrigger>
                  <SelectContent>
                    {unitOptions.map((option) => (
                      <SelectItem key={option.value} value={option.value}>
                        {option.label}
                      </SelectItem>
                    ))}
                  </SelectContent>
                </Select>
              )}
            </div>
          </div>
          
          <div className="pt-2 border-t">
            <div className="flex justify-between items-center">
              <span className="text-sm font-medium">Result:</span>
              <span className="text-xl font-bold font-mono">{result}</span>
            </div>
            <p className="text-sm text-muted-foreground mt-1">
              {inputValue && fromUnit && toUnit && unitOptions.length > 0 ? (
                <>
                  {inputValue} {unitOptions.find(option => option.value === fromUnit)?.label.split(' ')[0]} = {' '}
                  {result} {unitOptions.find(option => option.value === toUnit)?.label.split(' ')[0]}
                </>
              ) : ''}
            </p>
          </div>
        </div>
      </CardContent>
    </Card>
  );
}
